{%- if display_toc %}
    <h3>On This Page</h3>
    {{ toc }}
{%- endif %}
<h3>Contents</h3>
<ul>
    <li><a href="{{ pathto('index') }}">Home</a></li>
    <li><a href="{{ pathto('getting-started') }}">Getting Started</a></li>
    <li><a href="{{ pathto('changelog') }}">Changelog</a></li>
    <li><a href="{{ pathto('license') }}">License</a></li>
    <li><a href="{{ pathto('contact') }}">Contact / About</a></li>
</ul>
<hr>
<a href="{{ pathto('genindex') }}">Index</a>