SAMPLE_YOUTUBE_IDS = (
    "V84_F1QWdeU",
    "Urk3xn7l3AM",
    "0lKliaFllPA",
    "izEUig9w1tM",
    "n1DQ6uwfGE4",
    "N4I2If3x_9g",
    "aspMV6ERqpo",
    "9qcIsjrHENU",
    "euarux37A44",
    "VngstyKI754",
    "Gd9GOoITfXY",
    "wC6krCh48-Q",
    "2HN_xz_-pfg",
    "zGXpsJYNILg",
    "Rjm--7t8Llk",
    "jxJJkWHQF7k",
    "IBc1oVXen-o",
    "9Ix0o2iEcww",
    "4iMKd7LtgHQ",
    "YGDpfI7I04Y",
    "3tcYQE1B588",
    "vdV_PjTEaaY",
    "aumhEQtNyF4",
    "2Pe9ZLmvoDU",
    "4yZKGbq1YmA",
    "qmGOhT-8rJ4",
)

SAMPLE_SPOTIFY_IDS = (
    "3KLwjmeHEOg9x2ZfhbMUnW",
    "6z8lvJia3OzcIVCStbmEtU",
    "4eMenBlRap0b6FKsG6TWYg",
    "1LUh8e5N9lY7QJWi6ahJCo",
    "6MAszRR6tdDnMsjgVdw4Jh",
    "6dXQ2l97epGGwmdlBJQgj2",
    "7hzVvRkxjTwfR8Nlg1SvRR",
    "76wXKfIW9iwXtAUSkrf6xY",
    "7ufnUibHWOvxb8ROD4eALT",
    "5BkIWk6VYQCTZOQaRN4xa5",
    "0pgNYJjZFPlLssOzOVEdGa",
    "3LEhJxIYhjEL8oLjuqRXPE",
    "3hhZSVKfsAxqo4SphOKLqZ",
    "7acAKF2oEAjkzlYU8sRvbm",
    "7ipidZiUFsy1GVXpsoJGEC",
    "1GSUK92hQqWdOMNzs5j1IK",
    "5bHYUb0Gd1SFjmuhLzCcOQ",
    "6jqwwyNmeKzhU5B5dHoYsv",
    "2lRH5Y2FqjT2KZGW6cdP8R",
)
